# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#0!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
$Id: directindexer.py,v 1.19 2006/10/09 17:28:54 roeder Exp $
"""

import urlparse
import StringIO
import sys
from HarvestMan import harvestman
import xmlrpclib
import re
from re import DOTALL
__version__ = '1.0'

DEFAULT_MAXPAGES = 10000
DEFAULT_TIMEOUT = 5.0
DEFAULT_CHARSET = 'ISO-8859-15'
FIND_CHARSET_IN_PAGE_RE = r'.*<meta\s+http-equiv="Content-Type"\s+content="[a-zA-Z0-9_;\-/\s]+charset=(?P<charset>[a-zA-Z0-9_\-]+)".*'
FIND_CHARSET_IN_PAGE = re.compile(FIND_CHARSET_IN_PAGE_RE, DOTALL)
from indexexternalsite import Application
from HarvestMan.connector import HarvestManUrlConnector
from HarvestMan.common import Initialize as InitializeHarvestManUrlConnector
from HarvestMan.common import GetObject
from HarvestMan.pageparser import harvestManSimpleParser
from HarvestMan.rules import harvestManRulesChecker
from HarvestMan.urlparser import HarvestManUrlParser, HarvestManUrlParserError
from sgmllib import SGMLParseError

class DirectIndexer(Application):
    """Indexes directly in the catalog of a Plone site without
       going through XML-RPC"""
       
    def __init__(self, force, indexing_callback, pending_callback, site_url, time_limit, max_pages, charset, login=None, password=None, request_data=None, request_headers={}):
        """Building and setting options"""
        self.force=force # Do we force indexing even if not pending?
        # The site url is the starting point of the indexer. It is never changed, and will serve
        # later to see whether a link is followed or not. 
        self.site_url = site_url
        # Stop indexing after the time limit
        self.time_limit = int(time_limit)
        # Only index max max_pages pages
        self.max_pages = int(max_pages)
        #set regular expression for extractin title from html
        self.r = re.compile(r"(?si)<title\s*>(?P<title>.*)</title\s*>")
        self.indexing_callback = indexing_callback
        self.pending_callback = pending_callback
        self.indexed_pages = 0
        
        self.login = login
        self.password = password
        
        self.covered_links = {} # Keep track of links we have already seen
        self.out_of_scope_links = {} # Keep track of links pointing away from the site
        self.unreachable_links = {} # Keep track of what links couldn't be fetched
        self.indexed_links = {} # Keep track of what links were successfully indexed
        self.scheduled_links = {} # Keep track of what need to be fetched next
        self.original_links = {} # Keep track of what were the original links before adaptation
        self.charset = charset
        self.headers={}
        
        self.content_type_class = None
        self.content_type_type = None

        self.request_data = request_data
        self.request_headers = request_headers
        
    def harvestManMappingArgs(self, timelimit):
        """
        converting passed arguments to harvestman format
        """
        harvestMan_args=[]
        harvestMan_args.append('--verbosity=%(LEVEL)s' %{'LEVEL':5}) # O low, 5 high
        harvestMan_args.append('--basedir=downloaded_sites_plone')
        harvestMan_args.append('--maxfiles=%(NUM)s' %{'NUM':DEFAULT_MAXPAGES})
        harvestMan_args.append('--cache=no')
        harvestMan_args.append('--localize=no') # Don't need to change urls to be able to surf locally
        harvestMan_args.append('--retry=0') # Don't retry for failed urls
        harvestMan_args.append('--connections=1') # Only one network connection       
        harvestMan_args.append('--depth=10') # Limit the depth of urls to 10 (default)       
        harvestMan_args.append('--timelimit=%d' % timelimit) # Time limit in seconds       
        harvestMan_args.append('-S 0')
        harvestMan_args.append('--nocrawl')
        harvestMan_args.append('-f 0') # Only below this path
        harvestMan_args.append('--robots=no')

        harvestMan_args.append(self.site_url)
        """    
    -w, --workers=NUM           Enable worker threads and set the number of worker
                                threads to NUM. 
    -T, --maxthreads=NUM        Limit the number of tracker threads to NUM.
    -M, --maxfiles=NUM          Limit the number of files downloaded to NUM.
    -t, --timelimit=TIME        Run the program for the specified time TIME.    
    -R, --robots=yes/no         Enable/disable Robot Exclusion Protocol.
    -u, --urlfilter=FILTER      Use regular expression FILTER for filtering urls.
    --urlslist=FILE             Dump a list of urls to file FILE.
    --urltree=FILE              Dump a file containing hierarchy of urls to FILE.
        """


        return harvestMan_args

    def doXmlrpcIndexing(self, connector, urltofetch):
        pass # Ignore
        
    def doIndexing(self, connection, urltofetch):
        """sending xmlrpc data. connection, urltofetch are sent from harvestman's connector module.
        """
        data = connection.get_data()
        content_type_class, content_type_type = connection.get_content_type().split('/')
        
        if 1: # all verifications are made in harvestman module to optimize data indexing
           bindata=xmlrpclib.Binary(data)
           m = self.r.search(data)
           title = "empty title"
           if m:
               title = m.group('title').strip()
           bintitle=xmlrpclib.Binary(title)
           self.indexing_callback(urltofetch, bintitle, bindata)

    def run(self, timelimit=10):
        """Main entry after inits..."""
        self._log("Indexing site: %s" % self.site_url)
        link = self.site_url
        if link.endswith('/'):
            link = link[:-1]
        # Initiate the schedule, counting the level
        self.scheduled_links = {self.site_url.count('/'):[self.site_url,]}
        while self.scheduled_links and (self.pending_callback() or self.force):
            if self.indexed_pages >= self.max_pages:
                break
            
            # Give priority to pages close to the root
            min_depth=min(self.scheduled_links.keys())
            link = self.scheduled_links[min_depth].pop()
            if not self.scheduled_links[min_depth]:
                # List may be empty on that level, so destroy
                del self.scheduled_links[min_depth]
            
            if link.endswith('/'):
                link = link[:-1]
            if self.covered_links.has_key(link):
                continue
            else:
                self.covered_links[link]=1
            if self.index_url(link):
                if self.indexed_links.has_key(link):
                    self.indexed_links[link] = self.indexed_links[link]+1
                else:
                    self.indexed_links[link] = 1
            else:
                if self.unreachable_links.has_key(link):
                    self.unreachable_links[link] = self.unreachable_links[link]+1
                else:
                    self.unreachable_links[link] = 1
        
        return {'covered':self.covered_links,
                'out_of_scope':self.out_of_scope_links,
                'unreachable':self.unreachable_links,
                'indexed':self.indexed_links,
                'scheduled':self.scheduled_links,
                'original':self.original_links}        

    def get_charset_from_headers(self, http_headers):
        charset=None
        for http_header in http_headers:
            if http_header.lower() == 'content-type': # content="text/html;charset=ISO-8859-1"
                ctyp = http_headers[http_header]
                if ctyp.find(';') != -1:
                    charset = ctyp.split(';')[-1]
                    if charset.find('='):
                        charset=charset.split('=')[-1].strip().lower() #charset=ISO-8859-1
                break
        return charset

    def decode_html(self, conn, data):
        decoded = False
        # Step to decode data
        # 1- Use http header charset
        # 2- Use meta tag charset
        # 3- Use plone site charset
        # 4- Force with utf-8
        
        
        # 1- Use http header charset
        # 2- Use meta tag charset
        charset = self.get_charset_from_headers(conn.get_http_headers())
            
        if not charset:
            # If charset not defined http headers, search it meta tags
            # Search in the first 1000 octets
            buffer = data[:1000]
            match = FIND_CHARSET_IN_PAGE.match(buffer)
            
            if match is not None:
                charset = match.group('charset')
            
        if charset:
            try:
                data = data.decode(charset)
                decoded = True
            except:
                pass
        
        # 3- Use plone site charset
        if not decoded and self.charset:
            try:
                data = data.decode(self.charset)
                decoded = True
            except:
                pass            
                
        # 4- Force with utf-8
        if not decoded:
            data = data.decode('utf-8', 'replace')
            
        return data
        
    def index_url(self, url): 
        InitializeHarvestManUrlConnector()
        # Take and initialize the connector
        conn = HarvestManUrlConnector()
        harvestman.connector.HarvestManUrlConnector.doXmlrpcIndexing=self.doXmlrpcIndexing # adding  doXmlrpcIndexing method to harvestman
        try:
            hu = HarvestManUrlParser(url)
        except HarvestManUrlParserError:
            raise "Url can not be parsed" + url
        data = conn.url_to_data(url, login=self.login, password=self.password, request_data=self.request_data, request_headers=self.request_headers)
        self.data = data
        content_type = conn.get_content_type()
        if content_type:
            content_type_class, content_type_type = content_type.split('/')
        else:
            content_type_class = None
            content_type_type = None
 
        self.content_type_class = content_type_class
        self.content_type_type = content_type_type
     
        self.headers = conn.get_http_headers()            
        
        if data and content_type_class=='text' and content_type_type=='html':

            data = self.decode_html(conn, data)
                
            self.indexed_pages = self.indexed_pages + 1
            m = self.r.search(data)
            title = "Empty title."
            if m:
                title = m.group('title').strip()
            self.indexing_callback(url, title, data)
            if self.indexed_links.has_key(url):
                self.indexed_links[url] = self.indexed_links[url]+1
            else:
                self.indexed_links[url] = 1

            # Extract candidate links from html      
            candidate_links = self.fetch_candidate_links(data, hu)
            accepted_links = self.remove_out_of_scope_links(candidate_links)
            
            for link in accepted_links:
                if link.endswith('/'):
                    link = link[:-1]
                if not link in self.covered_links:
                    link_depth = link.count('/')
                    if self.scheduled_links.has_key(link_depth):
                        if not link in self.scheduled_links[link_depth]:
                            # Link is inserted at the beginning, so links encountered earlier are taken first
                            self.scheduled_links[link_depth].insert(0,link)
                    else:
                        # Initialize schedule for this level
                        self.scheduled_links[link_depth]=[link]
            return 1
        else:
            return self.index_url_non_html(conn, data)
        return 0

    def index_url_non_html(self, conn, data):
        return 0 # Don't treat non-html

    def get_links(self, data):
        links = []
        try:
            p=harvestManSimpleParser()
            p.handled = { 'a' : (('href', 'normal'),),
                          #('href', 'anchor')
                          'base': (('href', 'base'),),
                          'frame': (('src', 'normal'),),
                          #'img' : (('src', 'image'),),
                          #'form' : (('action', 'form'),),
                          #'link' : (('href', ''),),
                          #'body' : (('background', 'image'),),
                          #'script' : (('src', 'javascript'),),
                          #'applet' : (('codebase', 'appletcodebase'), ('code', 'javaapplet'))
                    }        
            p._anchors = False
            p.reset()
            p.feed(data)
            p.close()
            links=p.links
        except (SGMLParseError, IOError), e:
            pass
        return links
        
    def fetch_candidate_links(self, data, hu):
        links = self.get_links(data)
        urlobjlist=[]
        for typ, url in links:
            self.original_links[(typ, url)]=1
            is_cgi, is_php = False, False                
            if url.find('php?') != -1: 
                is_php = True
            if typ == 'form' or is_php:
                is_cgi = True
            if not url: 
                #print "Not url", url
                continue
            try:
                urlobjlist.append(HarvestManUrlParser(url, typ, is_cgi, hu))
            except HarvestManUrlParserError:
                continue
        return urlobjlist

    def remove_out_of_scope_links(self, urlobjlist):
        # Refuse out of scope links and accept only relative links or absolute links on the same base url
        accepted_links=[]
        for item in urlobjlist:
            if item.url[0] == '/':
                # Easy dectection of relative links
                item.isrels = True
            if item.isrel:
                # Accept relative links directly
                accepted_links.append(item.get_full_url())
            else:
                link = item.get_url()
                if link.endswith('/'):
                    link = link[:-1]
                # Perhaps the link is absolute?
                if link.lower().startswith(self.site_url.lower()): # Lower because of http://www.site.De 
                    # Need to check whether links starts with the given site url
                    # We take both urls as lowered as sometimes definitions of external sites
                    # can be made with mixed case                    
                    accepted_links.append(link)
                else:
                    # The rest is sent to the out of scope links, which will help us to debug later
                    if self.out_of_scope_links.has_key(link):
                        self.out_of_scope_links[link]=self.out_of_scope_links[link]+1
                    else:
                        self.out_of_scope_links[link]=1
        return accepted_links
        
    def _log(self, message, from_verbosity=0):
        """Writes message to console if verbosity >= options.verbosity"""
        return

if __name__ == '__main__':
    def indexing_callback(urltofetch, bintitle, bindata):
        print urltofetch, bintitle, len(bindata.data)
    print DirectIndexer(indexing_callback, "http://localhost:8080/").run()
