# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#0!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
The console tool that reindexes a site

$Id: indexexternalsite.py,v 1.14 2006/06/01 16:48:53 roeder Exp $
"""

import optparse
import urlparse
import StringIO
import sys
from HarvestMan import harvestman
import xmlrpclib
import re
__version__ = '1.0'

DEFAULT_MAXPAGES = 10000
DEFAULT_TIMEOUT = 5.0
DEFAULT_CHARSET = 'ISO-8859-15'


class Application(object):
    """The main app class"""
    
    def __init__(self):
        """Building and setting options"""
        
        global __version__, DEFAULT_MAXPAGES, DEFAULT_TIMEOUT, DEFAULT_CHARSET,CONTENT_TYPES_TO_RETRIEVE
        usage = ("usage: %prog [options] site_url catalog_url\n\n"
                 "Indexing an external site in a Zope ExternalSiteCatalog\n\n"
                 "site_url    is the site you want to browse\n"
                 "catalog_url goes to the ExternalSiteCatalog object")
        parser = optparse.OptionParser(usage=usage, version='%prog: ' + __version__)
        parser.add_option('-e', '--empty', dest='emptyCatalog', action='store_true', default=False,
                          help="Clears the ExternalSiteCatalog content before indexing"
                          )
        parser.add_option('-u', '--user', dest='username', metavar='USERNAME',
                          help=("Provide a Zope username when indexing, otherwise"
                                " anonymous indexing is attempted. Note that"
                                " you must fill the -p/--password option too")
                          )
        parser.add_option('-p', '--password', dest='password', metavar='PASSWORD',
                          help=("Provide a password if any -u/--user option"
                                " is in use")
                          )
        parser.add_option('-m', '--maxpages', dest='maxpages', metavar='MAXPAGES',
                          default=DEFAULT_MAXPAGES, type='int',
                          help="Stop after indexing this amount of pages (default: %d)" % DEFAULT_MAXPAGES
                          )
        parser.add_option('-t', '--timeout', dest='timeout', metavar='TIMEOUT',
                          default=DEFAULT_TIMEOUT, type='float',
                          help="Connection time out in seconds (default: %d)" % DEFAULT_TIMEOUT
                          )
        parser.add_option('-c', '--charset', dest='charset', metavar='CHARSET', default=DEFAULT_CHARSET,
                          help=("The charset to be used if none is provided from source pages"
                                " (default: %s)") % DEFAULT_CHARSET
                          )
        parser.add_option('-v', '--verbose', dest='verbosity', action='count', default=2,
                          help="Adds verbosity for each '-v' option"
                          )
        parser.add_option('-q', '--quiet', dest='quiet', action="store_true", default=False,
                          help=("Logs nothing to the console except errors,"
                                " whatever '-v' options are provided")
                          )
        self.options, args = parser.parse_args()
        
        # Checking user AND password or none of those
        help_file = StringIO.StringIO()
        parser.print_usage(file=help_file)
        if self.options.username and not self.options.password:
            self.commandError("You should provide a password if a username is given",
                              help_file)
        
        # Checking the args
        if len(args) != 2:
            self.commandError("You should provide 2 URL arguments", help_file)
        self.site_url = args[0]
        self.catalog_url = args[1]
        for name, value in (('site_url', self.site_url),
                            ('catalog_url', self.catalog_url)):
            scheme = urlparse.urlparse(value)[0]
            if scheme not in ('http', 'https'):
                self.commandError("%s is not a valid value for %s"
                                  % (value, name),
                                  help_file)

        # Setting XMLRPC basic auth if required
        if self.options.username:
            comps = list(urlparse.urlparse(self.catalog_url))
            comps[1] = self.options.username + ':' +self.options.password + '@' + comps[1]
            self.catalog_url = urlparse.urlunparse(comps)
        
        self.zcatalog = xmlrpclib.ServerProxy(self.catalog_url)
        #set regular expression for extractin title from html
        self.r = re.compile(r"(?si)<title\s*>(?P<title>.*)</title\s*>")
        return

    def commandError(self, message, help_file):
        """Bad argument"""

        print message
        print help_file.getvalue()
        sys.exit(1)

    def harvestManMappingArgs(self):
        """
        converting passed arguments to harvestman format
        """
        harvestMan_args=[]
        harvestMan_args.append('--verbosity=%(LEVEL)s' %{'LEVEL':self.options.verbosity})
        harvestMan_args.append('--basedir=downloaded_sites')
        harvestMan_args.append('--maxfiles=%(NUM)s' %{'NUM':self.options.maxpages})
        harvestMan_args.append('--cache=no')
        harvestMan_args.append('-S 0')
        harvestMan_args.append('--configfile=config.xml')
        harvestMan_args.append(self.site_url)
        return harvestMan_args
        
    
    def doXmlrpcIndexing(self,connection, urltofetch):
        """sending xmlrpc data. connection, urltofetch are sent from harvestman's connector module.
        """
        data = connection.get_data()
        content_type_class, content_type_type = connection.get_content_type().split('/')
        
        if 1: # all verifications are made in harvestman module to optimize data indexing
           bindata=xmlrpclib.Binary(data)
           m = self.r.search(data)
           title = "empty title"
           if m:
               title = m.group('title').strip()
           bintitle=xmlrpclib.Binary(title)
           self.zcatalog.xmlRpcIndex(urltofetch, bintitle,bindata,self.options.charset)

    def test_function(self):
        return "ok"

    def run(self):
        """Main entry after inits..."""
        self._log("Indexing site: %s" % self.site_url)
        self._log("into ExternalSiteCatalog: %s" % self.catalog_url)
        harvestman.connector.HarvestManUrlConnector.doXmlrpcIndexing=self.doXmlrpcIndexing # adding  doXmlrpcIndexing method to harvestman
        sys.argv=[sys.argv[0]] # harvestman doesn't support unknown args, must remove all additionnal args from sys.argv
        sys.argv+=self.harvestManMappingArgs() # addinf harvestman arguments
        if self.options.emptyCatalog:
            self.zcatalog.xmlRpcClear()
        spider = harvestman.harvestMan()
        spider.run_projects()
        self.clean_temp(spider)

    def clean_temp(self,spider):
        """
        clean temporary files
        """
        import os
        for root, dirs, files in os.walk(spider._cfg['projdir'], topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        os.rmdir(spider._cfg['projdir'])
        os.remove(os.path.join(spider._cfg['basedir'], spider._cfg['project']+'.hbp'))
    
    def _log(self, message, from_verbosity=0):
        """Writes message to console if verbosity >= options.verbosity"""
        if self.options.quiet:
            return
        verbosity = self.options.verbosity or 0
        if verbosity >= from_verbosity:
            print message
        return

    def test(self):
        if False:
            p = Page('http://localhost:8080')
            print p.charset()
            print p.realUrl()
            print p.contentType()
            print p.body
            print p.digest()
        if True:
            self.zcatalog.xmlRpcClear()
            url = "http://www.yahoo.com/"
            title = "Yahoo !"
            html = '<html><head></head><body>one two three</body></html>'
            charset = DEFAULT_CHARSET
            self.zcatalog.xmlRpcIndex(url, title, html, charset)
            url = "http://www.yahoo.com/toto"
            title = "Yahoo !"
            html = '<html><head></head><body>four five six</body></html>'
            self.zcatalog.xmlRpcIndex(url, title, html, charset)

if __name__ == '__main__':
    Application().run()
