# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
External Site Catalog Task
"""
__version__ = "$Revision: 1.11 $"
# $Source: /cvsroot/ingeniweb/ExternalSiteCatalog/content/ExternalSiteCatalogTask.py,v $
# $Id: ExternalSiteCatalogTask.py,v 1.11 2006/07/27 13:14:40 clebeaupin Exp $
__docformat__ = 'restructuredtext'

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.PloneMaintenance.MaintenanceTask import MaintenanceTask
from Products.CMFCore import permissions as CCP
from DateTime import DateTime
from persistent.dict import PersistentDict

# Plone2 FTI
factory_type_information = {
  'id'             : 'ExternalSiteCatalogTask',
  'portal_type'    : 'ExternalSiteCatalogTask',
  'meta_type'      : 'ExternalSiteCatalogTask',
  'description'    : 'An ExternalSiteCatalogTask',
  'content_icon'   : 'task.gif',
  'product'        : 'ExternalSiteCatalogTask',
  'factory'        : 'addExternalSiteCatalogTask',
  'immediate_view' : 'ExternalSiteCatalogTask_edit2',
  'filter_content_types' : 0,
  'allowed_content_types': (),
  'actions': (),
  }
  

# The usual factory
def addExternalSiteCatalogTask(self, id, title = '', REQUEST = {}):
    """
    Factory method for an ExternalSiteCatalogTask object
    """
    obj = ExternalSiteCatalogTask(id, title)
    self._setObject(id, obj)
#    if REQUEST.has_key('RESPONSE'):
#        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')

class ExternalSiteCatalogTask(MaintenanceTask):
    """ExternalSiteCatalogTask class"""
    meta_type = "ExternalSiteCatalogTask"
    global_allow   = 0
    manage_options = MaintenanceTask.manage_options
    security = ClassSecurityInfo()

    security.declarePrivate('__init__')
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        self.id=id
        MaintenanceTask.__init__(self, id, title)
        self.setScript_name('crawlExternalSite')
        self._setProperty('max_duration', 1, 'int')
        self._setProperty('max_pages', 1, 'int')
        self._setProperty('url', 'http://', 'string')
        self._setProperty('login', '', 'string')
        self._setProperty('password', '', 'string')
        self._setProperty('charset', 'utf-8', 'string')
        self.stats = PersistentDict()
        
    security.declareProtected(CCP.ManagePortal, "edit")
    def edit(self, infos):
        self.login = infos['login']
        self.password = infos['password']
        if not infos['charset']:
            raise "charset is required"
        self.charset = infos['charset']
        
        last_execution_date = infos['last_execution_date']
        if not isinstance(last_execution_date, DateTime):
            last_execution_date = DateTime(last_execution_date)
        self.last_execution_date = last_execution_date
        self.max_duration = int(infos['max_duration'])
        self.max_pages = int(infos['max_pages'])
        self.setScheduled_hour(infos['scheduled_hour'])
        if infos['period']=='daily':
           self.setScheduled_day_of_month('*')
        elif infos['period']=='monthly':
           self.setScheduled_day_of_month(1)
        self.url = infos['url']
        self.title = infos['title']


    def isPending(self, date=None):
        # In ExternalSite Tasks, wa also take care about the maximum duration of a task, so even if it
        # is pending for PloneMaintenance, we have an additional restriction here.
        from Products.CMFCore.utils import getToolByName
        pe = getToolByName(self, 'portal_externalcatalog')
        scheduled_hour = int(self.getScheduled_hour())
        max_duration = int(self.getMax_duration())
        in_duration = pe.in_duration(DateTime(), scheduled_hour, max_duration)
        return in_duration and MaintenanceTask.isPending(self)
                
    def runScript(self, crawlExternalSite, force=0):
        # Calling the script in this isolated method makes it possible to call
        # it differently (for example with additional parameters) in inherited
        # classes.
        self.stats = crawlExternalSite(force,
                                       self.url, 
                                       self.isPending,
                                       self.getMax_duration(),
                                       self.getMax_pages(),
                                       self.getCharset(),
                                       self.getLogin(),
                                       self.getPassword())

    def getTaskId(self):
        return self.getId()

    def getTitle(self):
        return self.title

    def getUrl(self):
        return self.url

    def getPeriod(self):
        if self.scheduled_day_of_month=='*':
            return 'daily'
        elif self.scheduled_day_of_month!='*':
            return 'monthly'

    def getScheduled_hour(self):
        return int(self.scheduled_hour)

    def getMax_duration(self):
        return int(self.max_duration)

    def getMax_pages(self):
        return int(self.max_pages)

    def getLast_execution_date(self):
        return self.last_execution_date

    def getLogin(self):
        return self.login

    def getPassword(self):
        return self.password
        
    def getCharset(self):
        return self.charset
        
InitializeClass(ExternalSiteCatalogTask)
