# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module implements an external site configuration
"""
__version__ = "$Revision: 1.5 $"
# $Source: /cvsroot/ingeniweb/ExternalSiteCatalog/content/ExternalSiteCatalogTaskProperties.py,v $
# $Id: ExternalSiteCatalogTaskProperties.py,v 1.5 2006/05/31 14:35:49 roeder Exp $
__docformat__ = 'restructuredtext'

from Products.ExternalSiteCatalog.config import PROJECTNAME

# Zope imports
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import Role
from DateTime import DateTime

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.Archetypes.public import *
from Products.Archetypes.BaseObject import BaseObject
from Products.CMFCore.utils import getToolByName

def getExternalSiteCatalogTaskProperties(object, is_existing, task_id):
    """getExternalSiteCatalogTaskProperties(self, ) => temp property sheet

    This method will create a temporary prop sheet in memory
    It will return the object itself.

    It must be called with 'object', which must be the place where the pty sheet is called.
    Id est, our pty sheet will be called with object.absolute_url()/xxx.
    """
    # Basic type init
    o = ExternalSiteCatalogTaskProperties("", )
    o.initializeArchetype()

    # Special permission handling for anonymous user
    o.manage_role(
        "Anonymous",
        [
        CMFCorePermissions.AccessContentsInformation,
        CMFCorePermissions.View,
        CMFCorePermissions.ModifyPortalContent,
        ],
        )

    # Data population; we set methods on the underlying object,
    # based on the accessors.
    if is_existing:
        o.setTaskId(task_id)
        for field in o.Schema().filterFields():
            accessor = field.getAccessor(o)
            if not accessor:
                continue
            meth_name = accessor.__name__
            setattr(o, meth_name, getattr(object, meth_name))

    # Now, return the special object
    return o.__of__(object)     #.portal_url.getPortalObject()).__of__(object)

periodOptions = DisplayList((
      (     'daily', 'Daily'    )
    , (     'monthly', 'Monthly'    )))

StartHourOptions = IntDisplayList((
      (     0, '0:00'    )
    , (     1, '1:00'    )
    , (     2, '2:00'    )
    , (     3, '3:00'    )
    , (     4, '4:00'    )
    , (     5, '5:00'    )
    , (     6, '6:00'    )
    , (     7, '7:00'    )
    , (     8, '8:00'    )
    , (     9, '9:00'    )
    , (     10, '10:00'    )
    , (     11, '11:00'    )
    , (     12, '12:00'    )
    , (     13, '13:00'    )
    , (     14, '14:00'    )
    , (     15, '15:00'    )
    , (     16, '16:00'    )
    , (     17, '17:00'    )
    , (     18, '18:00'    )
    , (     19, '19:00'    )
    , (     20, '20:00'    )
    , (     21, '21:00'    )
    , (     22, '22:00'    )
    , (     23, '23:00'    )
    ))

ExternalSiteCatalogTaskPropertiesSchema = Schema((
    StringField('taskId',
        required=False,
        searchable=False,
        default = "",
        # either mailto, absolute url or relative url
        validators = (),
        widget = StringWidget(
            description = "",
            description_msgid = "help_task_id",
            label = "Task Id",
            label_msgid = "label_task_id",
            i18n_domain = "externalsitecatalog",
            visible={'view' : 'hidden',
                     'edit' : 'hidden'},),
        ),
    StringField('title',
        required=True,
        searchable=False,
        default = "",
        # either mailto, absolute url or relative url
        widget = StringWidget(
            label_msgid = "label_title",
            label = "Title",
            description_msgid = "help_title",
            description = "The title of the external site to be crawled.",
            i18n_domain = "externalsitecatalog")
        ),
    StringField('url',
        required=True,
        searchable=False,
        default = "http://",
        # either mailto, absolute url or relative url
        validators = (),
        widget = StringWidget(
            description_msgid = "help_url",
            description = "The url to the external site. Will only crawl pages under the given path.",
            label = "URL",
            label_msgid = "label_url",
            i18n_domain = "externalsitecatalog")
        ),
    StringField('period',
        required=False,
        searchable=False,
        default='daily',
        vocabulary=periodOptions,
        widget=SelectionWidget(
                label="Period",
                label_msgid="label_period",
                description="Period in which the external site should be crawled.",
                description_msgid="help_period",
                i18n_domain = "externalsitecatalog"),
         ),
    IntegerField('scheduled_hour',
                required=1,
                default=0,
                vocabulary=StartHourOptions,
                widget=SelectionWidget(
                    label="Start hour",
                    label_msgid="label_scheduled_hour",
                    description="Hour from which on the external site can be crawled.",
                    description_msgid="help_date_criteria_value",
                    i18n_domain="externalsitecatalog"),
                ),
    IntegerField('max_duration',
        required=False,
        searchable=False,
        default=1,
        widget=IntegerWidget(
                label="Max. duration in hours",
                label_msgid="label_max_duration",
                description="Maximum number of hours to crawl the site. Must be one hour or more.",
                description_msgid="help_max_duration",
                i18n_domain = "externalsitecatalog"),
         ),
    IntegerField('max_pages',
        required=False,
        searchable=False,
        default=1,
        widget=IntegerWidget(
                label="Max. Pages to index",
                label_msgid="label_max_pages",
                description="Maximum number of pages after which to stop crawling.",
                description_msgid="help_max_pages",
                i18n_domain = "externalsitecatalog"),
         ),
    DateTimeField('last_execution_date',
                  required=True,
                  searchable=False,
                  default_method=DateTime,
                  languageIndependent=True,
                  widget = CalendarWidget(
                        label="Last update",
                        label_msgid = "label_last_execution_date",
                        description= "Last time the site is has been crawled.",
                        description_msgid = "help_last_execution_date",
                        i18n_domain = "externalsitecatalog")
                  ),
    StringField('charset',
        required=True,
        searchable=False,
        widget = StringWidget(
            label = "Charset",
            label_msgid = "label_charset",
            description = "Expected charset of the site. Will respect explicit charset declarations if possible.",
            description_msgid = "help_charset",
            i18n_domain = "externalsitecatalog")
          ),
    StringField('login',
        required=False,
        searchable=False,
        widget = StringWidget(
            label = "Login",
            label_msgid = "label_login",
            description = "Login for basic authentication.",
            description_msgid = "help_login",
            i18n_domain = "externalsitecatalog")
          ),
    StringField('password',
        required=False,
        searchable=False,
        widget = StringWidget(
            label = "Password",
            label_msgid = "label_password",
            description = "Password for basic authentication.",
            description_msgid = "help_password",
            i18n_domain = "externalsitecatalog")
        ),
    ),)

class ExternalSiteCatalogTaskProperties(SimpleItem, BaseObject, Role.RoleManager):
    """Implement external site catalog task configuration"""

    archetype_name = "ExternalSiteCatalogTaskProperties"
    meta_type = "ExternalSiteCatalogTaskProperties"
    isPrincipiaFolderish = 0 
    schema = ExternalSiteCatalogTaskPropertiesSchema

    def Title(self, **args):
        return self.getTitle()

    def getPortalTypeName(self):
        return "Document" # Fake a content type

    def absolute_url(self):
        utool = getToolByName(self, 'portal_url')
        return utool()

registerType(ExternalSiteCatalogTaskProperties, PROJECTNAME)

