# -*- coding: utf-8 -*-
## ExternalSiteCatalog
## Index and search external sites in a Plone site
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
The ExternalSiteCatalog tool

$Id: tool.py,v 1.20 2006/07/27 13:14:40 clebeaupin Exp $
"""
import re
import os
import sys
import urlparse
import codecs
import unicodedata
from Products.CMFPlone import transaction
from types import UnicodeType

# Zope components
from DateTime import DateTime
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from AccessControl import Permissions, Unauthorized, getSecurityManager

from OFS.SimpleItem import SimpleItem
from Products.CMFCore import permissions as CCP

from Globals import InitializeClass

from Products.ZCatalog.ZCatalog import ZCatalog
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.interfaces.portal_catalog \
     import portal_catalog as ICatalogTool

from Products.ZCTextIndex.HTMLSplitter import HTMLWordSplitter
from Products.ZCTextIndex.Lexicon import CaseNormalizer
from Products.ZCTextIndex.Lexicon import Splitter
from Products.ZCTextIndex.Lexicon import StopWordRemover
from Products.ZCTextIndex.ZCTextIndex import PLexicon
from Products.ZCTextIndex.PipelineFactory import element_factory
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
# Product imports
from config import *
from Products.ExternalSiteCatalog.content import ExternalSiteCatalogTaskProperties

from Products.ExternalSiteCatalog.content.ExternalSiteCatalogTask import ExternalSiteCatalogTask

from Products.ExternalSiteCatalog.bin.directindexer import DirectIndexer

# Archetypes imports
from Products.Archetypes.public import BaseContentMixin, registerType, \
    MinimalSchema, Schema, ImageField

# New Schema 
schema = MinimalSchema.copy()
schema['id'].widget.visible = {'view': 'invisible', 'edit': 'invisible',}
schema['title'].widget.visible = {'view': 'invisible', 'edit': 'invisible',}

##strictly done for debug!!!
#import locale
#locale.setlocale(locale.LC_ALL, '')

modulesecurityinfo = ModuleSecurityInfo()

_zmi = os.path.join(PACKAGE_PATH, 'zmi')
skins= os.path.join(PACKAGE_PATH, 'skins')


# Portal type actions for tool
type_actions = (
    {
    'id'            : 'view',
    'name'          : 'View',
    'action'        : 'string:${object_url}/externalsitecatalog_management_form',
    'permissions'   : (CCP.ManagePortal, ),
    'category'      : 'object',
    'visible'       : 1,
    },
    {
    'id'            : 'edit',
    'name'          : 'Edit',
    'action'        : 'string:${object_url}/externalsitecatalog_management_form',
    'permissions'   : (CCP.ManagePortal, ),
    'category'      : 'object',
    'visible'       : 1,
    },
)

def modify_fti(fti):
    fti['actions'] = type_actions

###
## ZMI helper
###

modulesecurityinfo.declareProtected(Permissions.view_management_screens, 'ZMIData')
class ZMIData:
    
    __allow_access_to_unprotected_subobjects__ = True
    
    def __init__(self, catalog):
        """Constructor"""
        
        self.catalog = catalog
        return
    
    def listRoles(self):
        """A list of potential roles"""

        return self.catalog.validRoles()
    
    def errors(self):
        """Mapping of {field name: error message}"""
        
        errors = {}
        try:
            dummy = codecs.lookup(self.catalog.charset)
        except LookupError, e:
            errors['charset'] = "This charset is not supported"
        
        if self.catalog.roleForIndexing not in self.listRoles():
            errors['roleForIndexing'] = "This is not a valid role"
        return errors
    
    def pythonCommand(self):
        """The shell command that runs Python"""
        if sys.platform == 'win32':
            return '"' + sys.executable + '"'
        else:
            return sys.executable
    def indexingUtility(self):
        """The full path to the indexing utility"""
        full_path = os.path.join(PACKAGE_PATH, 'bin', 'indexexternalsite.py')
        if sys.platform == 'win32':
            return '"'+full_path+'"'
        else:
            return full_path
    
###
## Catalog construction ressources
###


# We currently only support Latin normalizer and splitter
class HTMLLatinNormalizerAndSplitter: 
    rx = re.compile(r"(?L)\w+")
    rxGlob = re.compile(r"(?L)\w+[\w*?]*") # See globToWordIds() ab
    removeHTML = (re.compile(r"<[^<>]*>"), re.compile(r"&[A-Za-z]+;"),)
    
    def _normalize(self, text):
        """Normalize text : returns an ascii text

        @param text: Text to normalize"""
        
        utext = text
        if type(text) not in (UnicodeType,):
            charset = DEFAULT_CHARSET 
            utext = text.decode(charset, "replace")
        ntext = unicodedata.normalize('NFKD', utext)
        atext = ntext.encode('ascii', 'ignore')
        atext = atext.lower()
        for rmHTML in self.removeHTML:
            atext = rmHTML.sub(' ', atext)
        return atext

    def process(self, lst):
        result = []
        for text in lst:
           text = self._normalize(text)
           result.extend(self.rx.findall(text))
        return result
    
    def processGlob(self, lst):
        result = []
        for text in lst:
           text = self._normalize(text)
           result.extend(self.rxGlob.findall(text))
        return result


try:
    element_factory.registerFactory("Word Splitter" , "ExternalSite HTML Latin normalizer and splitter", HTMLLatinNormalizerAndSplitter)
except ValueError:
    # in case the normalizer is already registred, ValueError is raised
    pass


modulesecurityinfo.declarePublic('SimpleRecord')
class SimpleRecord:
    """ record-like class """

    def __init__(self, **kw):
        self.__dict__.update(kw)

modulesecurityinfo.declarePublic('enumerateIndexes')
def enumerateIndexes():
    """Return a list of ( index_name, type, extra ) tuples for the initial index set."""

    plaintext_extra = SimpleRecord(lexicon_id='plaintext_lexicon',
                                   index_type='Okapi BM25 Rank'
                                   )
    htmltext_extra = SimpleRecord(lexicon_id='htmltext_lexicon',
                                  index_type='Okapi BM25 Rank'
                                  )
    latin_extra = SimpleRecord(lexicon_id='latin_lexicon',
                               index_type='Okapi BM25 Rank')

    return (
        ('PrincipiaSearchSource', 'ZCTextIndex', latin_extra),
        ('hostname', 'FieldIndex', None),
        )

modulesecurityinfo.declarePublic('enumerateLexicons')
def enumerateLexicons():
    """ZCTextIndex lexicons"""

    return (
        ('plaintext_lexicon',
         Splitter(),
         CaseNormalizer(),
         StopWordRemover()
         ),
        ('htmltext_lexicon',
         HTMLWordSplitter(),
         CaseNormalizer(),
         StopWordRemover(),
         ),
        ('latin_lexicon',
         HTMLLatinNormalizerAndSplitter(),
         None,
         None
         ),
        )


modulesecurityinfo.declarePublic('enumerateColumns')
def enumerateColumns():
    """The catalog metadata definitions"""

    return ( 'title',
             'url')

def make_hostname(url):
    comps = urlparse.urlparse(url)
    # Remove potential credential (HTTP authentication)
    return comps[1].split('@')[-1].lower()

###
## External HTML page as fake Zope object
###
modulesecurityinfo.declarePublic('ExternalPage')
class ExternalPage(object):
    """Provides methods for cataloging"""

    def __init__(self, url, title, html_body, charset=None):
        self.url = url.strip()
        self.title = title.strip()
        self.PrincipiaSearchSource = html_body
        return
    
    def hostname(self):
        """hostname is an index"""
        return make_hostname(self.url)

class ExternalSiteCatalog(UniqueObject, ZCatalog, ActionProviderBase, BaseContentMixin):
    """A ZCatalog and services for indexing non Zope pages"""

    plone_tool = True
    id = 'portal_externalcatalog'
    title = "Misc utilities for ExternalSiteCatalog"
    portal_type = "ExternalSiteCatalog"
    meta_type = "ExternalSiteCatalog"
    global_allow = False
    schema = schema
    default_view = 'externalsitecatalog_management_form'
    immediate_view = 'externalsitecatalog_management_form'
    suppl_views = ()
    include_default_actions = 0

    _properties=(
        {'id':'title', 'type': 'string', 'mode':'w'},)
        

    _actions = ()

    manage_options = (ActionProviderBase.manage_options + \
                      SimpleItem.manage_options)

    __implements__ = (ICatalogTool, ZCatalog.__implements__,
                      ActionProviderBase.__implements__)


    security = ClassSecurityInfo()

    referenceable_types = {}

    manage_options = ( ZCatalog.manage_options +
                      ActionProviderBase.manage_options +
                      ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,
                     ))

    def __init__(self):
        ZCatalog.__init__(self, self.getId())
        BaseContentMixin.__init__(self, self.getId())
        self._initIndexes()

    def _initIndexes(self):
        # Setting the vocabulary
        if not hasattr(self, 'Vocabulary'):
            from Products.PluginIndexes.TextIndex.Vocabulary import Vocabulary
            vocabulary = Vocabulary('Vocabulary', 'Vocabulary', globbing=1)
            self._setObject('Vocabulary', vocabulary)

        # ZCTextIndex lexicons
        for id, splitter, normalizer, sw_remover in enumerateLexicons():
            pipelines = [x for x in (splitter, normalizer, sw_remover) if x is not None]
            lexicon = PLexicon(id, '', *pipelines)
            self._setObject(id, lexicon)

        # Content indexes
        self._catalog.indexes.clear()
        for index_name, index_type, extra in enumerateIndexes():
            self.addIndex(index_name, index_type, extra=extra)

        # Cached metadata
        self._catalog.names = ()
        self._catalog.schema.clear()
        for column_name in enumerateColumns():
            self.addColumn(column_name)

        # Misc properties
        self.roleForIndexing = DEFAULT_INDEXING_ROLE
        self.charset = DEFAULT_CHARSET
        return

    security.declarePublic('xmlRpcIndex')
    def xmlRpcIndex(self, url, title, html, charset):
        """Indexing a new page, typically invoked from XML-RPC"""

        self._checkXmlRpcRole()
        ep = ExternalPage(url, title.data, html.data, "latin-1")
        self.catalog_object(ep, uid=url)
        return

    security.declarePrivate('directIndex')
    def directIndex(self, url, title, html):
        """Indexing a new page, typically invoked directly"""

        utool = getToolByName(self, 'plone_utils')
        site_encoding = utool.getSiteEncoding()
        
        # Encode in site charset. Expecting to find UTF-8 here!
        html = html.encode(site_encoding, 'replace')
        
        ep = ExternalPage(url, title, html)
        self.catalog_object(ep, uid=str(url)) # Need to do a str(url) because the url can be in UTF-8, which is not accepted as a uid!

        ctool = getToolByName(self, 'portal_catalog')
        ctool.catalog_object(ep, uid=str(url)) # Need to do a str(url) because the url can be in UTF-8, which is not accepted as a uid!
        return

    security.declareProtected(CCP.ManagePortal, "indexSite")
    def indexSite(self, force, site, pending_callback, time_limit, max_pages, charset, login=None, password=None):
        return DirectIndexer(force, 
                             self.directIndex, 
                             pending_callback,
                             site, 
                             int(time_limit),
                             int(max_pages),
                             charset,
                             login,
                             password,).run()
        
    security.declarePublic('xmlRpcClear')
    def xmlRpcClear(self):
        """Cleaning up the catalog"""

        self._checkXmlRpcRole()
        self.manage_catalogClear()
        return

    def _checkXmlRpcRole(self):
        """Checking access control"""

        user = getSecurityManager().getUser()
        if not user.has_role(self.roleForIndexing, object=self):
            raise Unauthorized, 'You are not allowed to index anything'
        

    security.declareProtected(Permissions.view_management_screens, 'manage_externalSiteCatalog')
    manage_externalSiteCatalog = PageTemplateFile(
        os.path.join(_zmi, 'manage_externalSiteCatalog'), GLOBALS)
    
    security.declareProtected(Permissions.view_management_screens, 'manage_externalSiteCatalogData')
    def manage_externalSiteCatalogData(self):
        """Provides data to the ZMI form"""
        return ZMIData(self)
    
    security.declareProtected(Permissions.view_management_screens, 'manage_editExternalSiteCatalog')
    def manage_editExternalSiteCatalog(self, roleForIndexing, charset, REQUEST=None):
        """Handles the ZMI form"""

        self.roleForIndexing = roleForIndexing
        self.charset = charset
        if REQUEST is not None:
            message = "Changes saved"
            return self.manage_externalSiteCatalog(self, REQUEST, manage_tabs_message=message,
                                                   management_view='External site')
        return
    
    ###
    ## PMI
    ###

    security.declareProtected(CCP.ManagePortal, "getExternalSitesInfo")
    def getExternalSitesInfo(self):
        """Returns a list with information on maintenance tasks for external
           sites. The list contains dictionaries with the id, title, last_execution_date
           and nex_update keys.
        """
        pm = getToolByName(self, 'portal_maintenance')
        tasks=[]
        for task in pm.tasks.objectValues():
            if task.meta_type=='ExternalSiteCatalogTask':
                info={}
                info["id"]=task.getId()
                info["title"]=task.title
                info["last_execution_date"]=task.getLastExecutionTime()

                current_date=DateTime()
                next = task.getNextScheduledExecutionTime(current_date)
                
                if next.day() == current_date.day() and next.month() == current_date.month():
                    info["next_update"]=['today',]
                else:
                    if task.getPeriod()=='monthly':
                        info["next_update"]=['next month',]
                    else:
                        info["next_update"]=['tomorrow',]
                
                if task.isPending():
                    info["is_pending"]=1
                    # Override if is pending
                    info["next_update"]=['today',]
                else:
                    info["is_pending"]=0
                    
                info["next_update"].append(int(task.getScheduled_hour()))
                info["next_update"].append(int(task.getMax_duration()))
                    
                info["pages_in_catalog"]=len(self.searchResults(hostname=make_hostname(task.url)))
                stats={}
                for key in task.stats.keys():
                    stats[key]=task.stats[key]
                info["stats"]=stats
                info["max_duration"]=task.getMax_duration()
                info["max_pages"]=task.getMax_pages()
                info["url"]=task.getUrl()
                info["charset"]=task.getCharset()
                info["period"]=task.getPeriod()
                info["login"]=task.getLogin()
                info["password"]=task.getPassword()
                info["scheduled_hour"]=task.getScheduled_hour()
                
                tasks.append(info)
        return tasks
        
    security.declareProtected(CCP.ManagePortal, "addExternalSiteCatalogTask")
    def addExternalSiteCatalogTask(self, infos):
        pm = getToolByName(self, 'portal_maintenance')
        pe = getToolByName(self, 'portal_externalcatalog')
        
        # Get a unique id in the tasks folder
        id =taskId = baseTaskId ='externalsitecatalogtask'
        existing_ids = pm.tasks.objectIds()
        i=0
        while taskId in existing_ids:
            i=i+1
            taskId = baseTaskId + str(i)

        infos['taskId']=taskId
        obj = ExternalSiteCatalogTask(infos['taskId'], infos['title'])
        pm.tasks._setObject(infos['taskId'], obj)
        task = self.getExternalSiteCatalogTask(infos['taskId'])
        self.editExternalSiteCatalogTask(infos)

    security.declareProtected(CCP.ManagePortal, "editExternalSiteCatalogTask")
    def editExternalSiteCatalogTask(self, infos):
        task = self.getExternalSiteCatalogTask(infos['taskId'])
        task.edit(infos)

    def resetCatalogForSite(self, remote_url):
        # Removes all content of the site in the catalog
        
        # First, we look for everything that was indexed for this hostname
        brains=self.searchResults(hostname=make_hostname(remote_url))
        # We need to store everything we want to delete in a separate list, otherwise uncataloging won't work due to laziness
        brains_to_delete = []
        for brain in brains:
            # We only take what is below our part of the site
            if brain.url.lower().startswith(remote_url.lower()):
                # Add to list for deletion. Can't remove from catalog here as brain is lazy
                brains_to_delete.append(brain)
        for brain in brains_to_delete:
            # Finally we can remove the brain from the catalog
            self.uncatalog_object(brain.getPath())
                
    security.declareProtected(CCP.ManagePortal, "deleteExternalSiteCatalogTasks")
    def deleteExternalSiteCatalogTasks(self, ids):
        # First delete site from catalog
        for task_id in ids:
            task = self.getExternalSiteCatalogTask(task_id)
            self.resetCatalogForSite(task.getUrl())
        # Then remove task from maintenance tool
        pm = getToolByName(self, 'portal_maintenance')
        pm.tasks.manage_delObjects(ids)

    security.declareProtected(CCP.ManagePortal, "indexExternalSiteCatalogTasks")
    def indexExternalSiteCatalogTasks(self, ids):
        pm = getToolByName(self, 'portal_maintenance')
        for task_id in ids:
            task = self.getExternalSiteCatalogTask(task_id)
            self.resetCatalogForSite(task.getUrl())
            # Always runs task
            task.runTask(force=1)
            transaction.commit()
            
    security.declareProtected(CCP.ManagePortal, "getExternalSiteCatalogTask")
    def getExternalSiteCatalogTask(self, task_id):
        pm = getToolByName(self, 'portal_maintenance')
        return getattr(pm.tasks, task_id)

    security.declareProtected(CCP.ManagePortal, "hasExternalSiteCatalogTask")
    def hasExternalSiteCatalogTask(self, task_id):
        pm = getToolByName(self, 'portal_maintenance')
        if task_id in pm.tasks.objectIds():
            return True
        else:
            return False

    security.declareProtected(CCP.ManagePortal, "getPropertySheet")
    def getPropertySheet(self, task_id):
        """getPropertySheet(self, task_id) => temp property sheet

        This method will create a temporary prop sheet in memory
        It will return the object itself.
        """
        pu = getToolByName(self, 'portal_url')
        if task_id:
            target = self.getExternalSiteCatalogTask(task_id=task_id).__of__(self)
            is_existing=1
        else:
            target = pu.getPortalObject().__of__(self)
            is_existing=0
        return ExternalSiteCatalogTaskProperties.getExternalSiteCatalogTaskProperties(
            target,
            is_existing,
            task_id
            )

    def in_duration(self, date, scheduled_hour, duration): 
        if duration==0:
            return 0
        if scheduled_hour + duration > 24:
            return 1
        if scheduled_hour + duration > date.hour():
            return 1
        else:
            return 0

registerType(ExternalSiteCatalog)
